#pragma once

using namespace System;
using namespace System::Runtime::InteropServices;

namespace ProtocolPlugin
{
	// This client-implemented managed interface is called each time the new request must be processed.
	public interface class iProtoPlug
	{
	public:
		// Is called by unmanaged code when the new request should be processed.
		// You must return text/html data encoded in UTF-8.
		// The return value is the document length in bytes.
		// 'url' is the full URL requested by the browser, including the protocol part, and all parameters.
		int getStream( System::String ^url, [Out] System::IO::Stream  ^ % objStream );
	};

	// This managed class represents the main DLL interface.
	// Example usage scenario:
	//
	// * Create the 'ProtocolPlugin.ProtoPlug' instance.
	//
	// * Respond on the iProtoPlug.getStream call:
	//      1. Construct 'new StreamWriter( new MemoryStream(), Encoding.UTF8 )'
	//      2. Write the document content, appropriate for the provided URL.
	//      3. Call StreamWriter.Flush()
	//      4. StreamWriter.BaseStream.Seek( 0, SeekOrigin.Begin )
	//      5. return StreamWriter.BaseStream.Length
	public ref class ProtoPlug: System::IDisposable
	{
	private:
		// pointer to the IClassFactory that constructs 'CInternetProtocol' objects
		IClassFactory *m_pClassFactory;
	public:
		// 'strProtocol' is the fraction of the URLs that comes before the "://" characters.
		// 'handler' is the managed client-implemented object that generates the HTML content when requested.
		ProtoPlug( System::String ^strProtocol, iProtoPlug ^handler );

		// Destructor i.e. IDisposable.Dispose()
		virtual ~ProtoPlug();
	};
}
